// Zisterne - angepasste Füllstandsanzeige nach Reichelt und YT-Beiträgen
// Bearbeitet von Heimwerker Leopoldi www.max-mg.de  März 2024
// Uno/Nano, Schallsensor: HC-SR04, Display:OLED 0.96" 128x64, 2 Stück DS18B20

#include <NewPing.h>              // Ultraschall-Sensor
#include <OneWire.h>              //fuer 1-Wire-Bus
#include <DallasTemperature.h>    // TemperaturSensoren  DS18B20
#include <Adafruit_I2CDevice.h>   //I2C wie Wire.h für Display
#include <Adafruit_GFX.h>         //fuer OLED
#include <Adafruit_SSD1306.h>     //fuer OLED

#define LEVELFULL 90      // Hoehe Wassersaeule wenn voll (=100%) in cm
#define LEVELEMPTY 100    // Abstand Sensor zu Boden in cm
#define VOLUME  900       // Maximales Wasservolumen in l iter

#define ONE_WIRE_BUS 10   //PIN definieren für 1-Wire-Bus
#define OLED_ADDRESS 0x3C // I2C-Adresse des Displays 0x3C, 0x3D, .... (siehe Datenblatt)
#define OLED_RESET     4  // Reset pin
#define SCREEN_WIDTH 128  // OLED Display Breite in Pixel
#define SCREEN_HEIGHT 64  // OLED Display Hoehe in Pixel
#define trigPin  2        // Trigger Pin
#define echoPin  3        // Echo Pin
#define MAX_DISTANCE 320  // Begrenzung fuer Ultraschallsensor

// Kommentar: SCL auf A5   I2C-modul Wire.h bzw. Adafruit_..
//            SDA auf A4
//            1-Wire-Bus auf Port D10 für DS18B20

OneWire oneWire(ONE_WIRE_BUS);        // 1-Wire-Bus instanzieren
DallasTemperature sensors(&oneWire);  // Instanzieren der Dallas Sensoren

float duration;
int lefty, distance, levelact, levelpercent, volumeact;

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

NewPing sonar(trigPin, echoPin, MAX_DISTANCE); // NewPing Setup

void setup() {
  Serial.begin(9600); // nur fuer Debugzwecke
  sensors.begin();

  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, OLED_ADDRESS)) { 
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Endlosschleife
  }  
}

void testausgabeMonitor(float tempC1, float tempC2) {
         Serial.println("---");
         Serial.print("distance: Abstand: Sensor-Wasserstand  ");
         Serial.print(distance);
         Serial.println(" cm");
         Serial.print("tempC1: Temperatur in Zisterne  ");
         Serial.println(tempC1);
         Serial.print("tempC2: Außenfühler  ");
         Serial.println(tempC2);
         Serial.print("levelpercent: Volumen in Prozent  ");
         Serial.println(levelpercent);
         Serial.print("volumeact: Volumen in Litern  ");
         Serial.println(volumeact);
         Serial.println();
       //delay(50);
}

void loop() {
  delay(10); // Wartezeit zwischen Pings (ca. 20 pings/sec). nicht kleiner als 29ms! 

  // Temperaturen abfragen
  sensors.requestTemperatures();
  // Sensor 0
  float tempC1 = sensors.getTempCByIndex(0);
  float tempC2 = sensors.getTempCByIndex(1);
  //Messung mit Mittelung
  int iterations = 5;
  duration = sonar.ping_median(iterations);      //duration = sonar.ping() ohne Mittelung
//distance = int((duration / 2) * 0.0343);       //Abstand in cm
  distance = int((duration / 2) * (0.03313 + 0.0000606 * tempC1)); //Temperautrkompensiert
                                  //Temperautrkompensiert nach YT:  Bitbastelei#327
 //++++++++++
  testausgabeMonitor(tempC1, tempC2); 
 //++++++++++
 
  //Fuellstand berechnen
  levelact = (LEVELEMPTY - distance);

  //Fuellstand in Prozent berechnen
  levelpercent = (levelact * 100 / LEVELFULL);
  
  //Berechnung Volumen
  volumeact = ((VOLUME / 100) * levelpercent);

  display.clearDisplay(); // Clear display buffer

  //Ueberschrift
  display.setTextSize(2);      // Schriftgroesse
  display.setTextColor(WHITE); // Textfarbe
  display.setCursor(0, 0);     // obere linke Ecke
  display.print("Fuellstand");
  
  //Fuellstand in Prozent
  display.setTextSize(2);      // Schriftgroesse
  display.setTextColor(WHITE); // Textfarbe
  display.setCursor(50, 24);   // obere linke Ecke
  display.print(levelpercent);
  display.print(" %");
  
  //Fuellstand in Litern
  display.setTextSize(2);
  display.setCursor(50, 48);
  display.print((volumeact));  
  display.print(" l");    
  
  //Balkenanzeige
  levelpercent = constrain(levelpercent, 0, 100);  //um nur Werte zwischen 0 und 100 zu erhalten
  lefty = map(levelpercent, 0, 100, 63, 16);       //Hoehe des Balkens ermitteln
  
  //aeusserer Rahmen zeichnen
  display.drawRect(0, 16, 38, 48, WHITE);
  
  //Balken zeichnen
  display.fillRect(0, lefty, 38, 48, WHITE);
  
  //Display anzeigen
  display.display();
}
