// Zisterne - angepasste Füllstandsanzeige nach Reichelt und YT-Beiträgen
// Bearbeitet von Heimwerker Leopoldi www.max-mg.de  März 2024
// Uno/Nano, Schallsensor: HC-SR04, Display: OLED 1,3" 128x64, 1/2 Stück DS18B20

#include <NewPing.h>              // Ultraschall-Sensor
#include <Wire.h>                 // I2C Lib: SCL auf A5, SDA auf A4
#include <DallasTemperature.h>    // TemperaturSensoren  DS18B20
#include <Arduino.h>
#include <U8g2lib.h>

U8G2_SH1106_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);

void u8g2_prepare(void) {
  u8g2.setFont(u8g2_font_7x13_tf);  // u8g2_font_7x13_tf
  u8g2.setFontRefHeightExtendedText();
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setFontDirection(0);
}

void draw(float tempC1, int levelpercent, int volumeact)  {    //float tempC1, float levelpercent, float volumeact) 
  u8g2_prepare();
  u8g2.drawStr( 0, 0, "ZisterneFuellstand");

  u8g2.setCursor(70, 18);     
  u8g2.print(tempC1,1);       // Zisternen Lufttemperatur
  u8g2.setCursor(70, 37);
  u8g2.print(levelpercent);   // Zisternen Füllstand in %  
  u8g2.setCursor(70, 54);
  u8g2.print(volumeact,1);   // Füllstand in Litern

  u8g2.drawGlyph( 52, 18,0x003c);   // < 
  u8g2.drawGlyph(110, 18,0x00b0);   // °
  u8g2.drawStr(118, 18, "C");
  u8g2.drawStr(110, 37, "%");
  u8g2.drawStr(110, 54, "l");
}
  
// Zisterne Parameter
#define LEVELFULL 90  //75  //Hoehe Wassersaeule wenn voll (=100%) in cm
#define LEVELEMPTY 100 //180 //Abstand Boden zu Sensor in cm
#define VOLUME  900   //3500   //Volumen des zu messenden Mediums bei 100% in l

// Wirebus
#define ONE_WIRE_BUS 10   // 1-Wire-Bus auf Port D10 für DS18B20
#define trigPin  2        // Trigger Pin
#define echoPin  3        // Echo Pin
#define MAX_DISTANCE 320  // Begrenzung fuer Ultraschallsensor
OneWire oneWire(ONE_WIRE_BUS);        // 1-Wire-Bus instanzieren
DallasTemperature sensors(&oneWire);  // Instanzieren der Dallas Sensoren
float duration;
int lefty, distance, levelact, levelpercent, volumeact;
NewPing sonar(trigPin, echoPin, MAX_DISTANCE); // NewPing Setup


void setup() {
  u8g2.begin();
  sensors.begin();
  Serial.begin(9600); // nur fuer seriellen Monitor in der IDE
}  

void testanzeige(float tempC1, float tempC2, float volumeact) {
  //Test serielle Ausgabe
         Serial.println("---");
      Serial.print("distance: Abstand: Sensor-Wasserstand  ");
         Serial.print(distance);
         Serial.println(" cm");
   
      Serial.print("Temperature:   ");
         Serial.print(tempC1);
         Serial.println("°C");
      //   Serial.print("         Außenfühler: "); // keine OLEd Ausgabe!
      //   Serial.print(tempC2);
      //   Serial.println("°C");
                  
      Serial.print("levelpercent: Volumen in Prozent  ");
         Serial.println(levelpercent);
         Serial.println(volumeact);
        //delay(50);
   }

void loop() {
  delay(10); // Wartezeit zwischen Pings (ca. 20 pings/sec). nicht kleiner als 29ms! 

  
  sensors.requestTemperatures();  // Temperaturen abfragen
  float tempC1 = sensors.getTempCByIndex(0); // Sensor 0 - Master
//  float tempC2 = sensors.getTempCByIndex(1); // Sensor 1 - Slave
  //Messung mit Mittelung
  int iterations = 5;
  duration = sonar.ping_median(iterations);      //duration = sonar.ping() ohne Mittelung
  // distance = int((duration / 2) * 0.0343);       //Abstand in cm
  distance = int((duration / 2) * (0.03313 + 0.0000606 * tempC1)); //Temperautrkompensiert

 //*****wenn aktiv ist der Speicher zu klein ??!!!
 // testanzeige(tempC1, tempC2, volumeact);  //auf seriellen Monitor der IDE
 //*****
 
  //Fuellstand berechnen
  levelact = (LEVELEMPTY - distance);

  //Fuellstand in Prozent berechnen
  levelpercent = (levelact * 100 / LEVELFULL);
  lefty = map(levelpercent, 0, 100, 63, 16);       //Hoehe des Balkens ermitteln
  
  //Berechnung Volumen
  volumeact = ((VOLUME / 100) * levelpercent);

  u8g2.clearBuffer();
  u8g2_prepare();
  draw(tempC1, levelpercent, volumeact); //Anzeige Variablen und Maßeinheit

  //Balkenanzeige
  levelpercent = constrain(levelpercent, 0, 100);  //um nur Werte zwischen 0 und 100 zu erhalten
  lefty = map(levelpercent, 0, 100, 0, 46);       //Hoehe des Balkens ermitteln
 
  //auesserer Rahmen zeichnen 
  u8g2.drawRFrame(0, 14,40,50, 0);       // Rahmen  x,-y, B , H

  //Balken zeichnen
  u8g2.drawBox(2, 16+46-lefty, 36, lefty);  //gefüllte Box zeichen
           //   x vl, -y vo, Rechteck B, -H

   u8g2.sendBuffer();
 delay(100);
}
  
